/* abc - The AspectBench Compiler
 * Copyright (C) 2007 Eric Bodden
 *
 * This compiler is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This compiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this compiler, in the file LESSER-GPL;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package abc.tm.weaving.weaver.tmanalysis;

import java.util.HashSet;
import java.util.Set;

import soot.Unit;
import soot.jimple.Stmt;
import soot.toolkits.graph.DirectedGraph;
import soot.toolkits.scalar.ForwardFlowAnalysis;
import abc.da.weaving.weaver.depadviceopt.ds.Shadow;
import abc.tm.weaving.weaver.tmanalysis.stages.AnalysisJob;

/**
 * For each unit in a given unit graph, computes the set of active shadows that may reach this unit, i.e. may
 * precede this unit in the execution. The before-flow excludes shadows at the current unit, while the after-flow
 * includes them.
 *
 * @author Eric Bodden
 */
public class ReachingActiveShadowsAnalysis extends ForwardFlowAnalysis<Unit,Set<Shadow>> {

	protected final AnalysisJob job;

    /**
     * @param graph unit graph for a given method
     * @param tm tracematch for which to compute reaching active shadows
     */
    public ReachingActiveShadowsAnalysis(DirectedGraph<Unit> graph, AnalysisJob job) {
        super(graph);
        this.job = job;
        doAnalysis();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void flowThrough(Set<Shadow> in, Unit unit, Set<Shadow> out) {
        out.clear();
        out.addAll(in);
        out.addAll(job.shadowsOfStmt((Stmt) unit));
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void copy(Set<Shadow> source, Set<Shadow> dest) {
        dest.clear();
        dest.addAll(source);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Set<Shadow> entryInitialFlow() {
        return new HashSet<Shadow>();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void merge(Set<Shadow> in1, Set<Shadow> in2, Set<Shadow> out) {
        out.clear();
        out.addAll(in1);
        out.addAll(in2);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Set<Shadow> newInitialFlow() {
        return new HashSet<Shadow>();
    }

   
}
