package abc.tm.weaving.weaver.tmanalysis;

import java.util.HashMap;
import java.util.Map;

import soot.SootMethod;
import abc.da.weaving.weaver.depadviceopt.ds.Shadow;
import abc.main.Main;
import abc.tm.weaving.aspectinfo.PerSymbolTMAdviceDecl;
import abc.tm.weaving.aspectinfo.TMGlobalAspectInfo;
import abc.tm.weaving.aspectinfo.TraceMatch;


public class SymbolNames {

	protected HashMap<Shadow, String> shadowToSymbolName;
	protected HashMap<SootMethod, TraceMatch> adviceMethodToTraceMatch;
	
    private SymbolNames() { 
    	shadowToSymbolName = new HashMap<Shadow, String>();
		adviceMethodToTraceMatch = new HashMap<SootMethod, TraceMatch>();
		
		TMGlobalAspectInfo gai = (TMGlobalAspectInfo) Main.v().getAbcExtension().getGlobalAspectInfo();
		for (TraceMatch tm : gai.getTraceMatches()) {
			for (String tmSymbol : tm.getSymbols()) {
				adviceMethodToTraceMatch.put(tm.getSymbolAdviceMethod(tmSymbol), tm);
			}
		}
    }
    
	public String symbolNameForShadow(Shadow s) {
		if(!(s.getAdviceDecl() instanceof PerSymbolTMAdviceDecl))
			return null;
	
		String symbolName = shadowToSymbolName.get(s);
		if(symbolName==null) {
			//compute symbol name
			SootMethod adviceMethod = s.getAdviceDecl().getImpl().getSootMethod();
			TraceMatch tm = adviceMethodToTraceMatch.get(adviceMethod);
	        Map<SootMethod,String> adviceMethodToSymbol = new HashMap<SootMethod, String>();
	        for (String sym : tm.getSymbols()) {
				SootMethod am = tm.getSymbolAdviceMethod(sym);
				adviceMethodToSymbol.put(am,sym);
			}
	        Map<Shadow,String> shadowToSymbolName = new HashMap<Shadow, String>();
			
	        symbolName = adviceMethodToSymbol.get(adviceMethod);
			shadowToSymbolName.put(s, symbolName);
		}
		
		return symbolName;
	}
	
	public boolean isArtificialShadow(Shadow s) {
		String symbolName = symbolNameForShadow(s);
		return symbolName!=null && symbolName.equals("newDaCapoRun");
	}
	
	
	//singleton pattern
	
	protected static SymbolNames instance;

    public static SymbolNames v() {
		if(instance==null) {
			instance = new SymbolNames();
		}
		return instance;		
	}
	
	/**
	 * Frees the singleton object. 
	 */
	public static void reset() {
		instance = null;
	}

}
