package abc.tm.weaving.weaver.tmanalysis;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import soot.Local;
import soot.RefLikeType;
import soot.Unit;
import soot.jimple.Stmt;
import soot.jimple.toolkits.pointer.InstanceKey;
import abc.tm.weaving.aspectinfo.TraceMatch;
import abc.tm.weaving.matching.State;
import abc.tm.weaving.weaver.tmanalysis.ds.Configuration;
import abc.tm.weaving.weaver.tmanalysis.ds.Constraint;
import abc.tm.weaving.weaver.tmanalysis.ds.Disjunct;
import abc.tm.weaving.weaver.tmanalysis.ds.PreciseSymmetricDisjunct;
import abc.tm.weaving.weaver.tmanalysis.mustalias.InstanceKeyNonRefLikeType;
import abc.tm.weaving.weaver.tmanalysis.mustalias.TMFlowAnalysis;
import abc.tm.weaving.weaver.tmanalysis.stages.AnalysisJob;

public class TMWorklistBasedAnalysis extends WorklistAnalysis<Unit, Abstraction> implements TMFlowAnalysis {

	protected AnalysisJob job;
	
	public TMWorklistBasedAnalysis(AnalysisJob job, int maxJobCount) throws TimeOutException {
		super(job.unitGraph(),maxJobCount);
		this.job = job;
		
		Disjunct<InstanceKey> initialDisjunct = new PreciseSymmetricDisjunct(job);
		initialDisjunct.setFlowAnalysis(this);
		Constraint.initialize(initialDisjunct);
		
		doAnalysis();
	}

	@Override
	protected Set<Abstraction> initialConfigurations() {
        Set<Abstraction> compoundConfigs = new HashSet<Abstraction>();
        for (Iterator stateIter = job.stateMachine().getStateIterator(); stateIter.hasNext();) {
            State state = (State) stateIter.next();
            if(!state.isFinalNode()) {

                Configuration entryInitialConfiguration = new Configuration(
                		this,
                		Collections.singleton(state),
                		false //count final-hits, if not aborting when hitting final
                );

                //be conservative at the beginning: know nothing about any values being in the initial state        
        		HashSet<InstanceKey> valuesInInitialState = new HashSet<InstanceKey>();

                compoundConfigs.add(new Abstraction(this, entryInitialConfiguration,valuesInInitialState));
            }
        }
		return compoundConfigs;
	}
	
	public AnalysisJob getJob() {
		return job;
	}

	public TraceMatch getTracematch() {
		return job.traceMatch();
	}

	public void hitFinal() {
	}

	public Map<String, InstanceKey> reMap(Map<String, Local> bindings) {
        Map<String,Local> origBinding = bindings;
        Map<String,InstanceKey> newBinding = new HashMap<String, InstanceKey>();
        for (Map.Entry<String,Local> entry : origBinding.entrySet()) {
            String tmVar = entry.getKey();
            Local adviceLocal = entry.getValue();
            Stmt stmt = job.defStmtOf(adviceLocal); //may be null, if adviceLocal is not part of this method
            InstanceKey instanceKey = (adviceLocal.getType() instanceof RefLikeType) ?
              new InstanceKey(adviceLocal,stmt,job.method(),job.localMustAliasAnalysis(),job.localNotMayAliasAnalysis()) :
              new InstanceKeyNonRefLikeType(adviceLocal,stmt,job.method(),job.localMustAliasAnalysis(),job.localNotMayAliasAnalysis());
            newBinding.put(tmVar, instanceKey);
        }
        return newBinding;
	}

}
