/**
 * 
 */
package abc.tm.weaving.weaver.tmanalysis;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import soot.MethodOrMethodContext;
import soot.SootMethod;
import soot.Unit;
import soot.jimple.Stmt;
import soot.jimple.toolkits.callgraph.Edge;
import soot.jimple.toolkits.pointer.InstanceKey;
import abc.da.weaving.weaver.depadviceopt.ds.Shadow;
import abc.tm.weaving.weaver.tmanalysis.ds.Configuration;

public class Abstraction implements WorklistAnalysis.Config<Unit> {
	
	protected final TMWorklistBasedAnalysis analysis;

	protected final Configuration configuration;
	
	protected final Set<InstanceKey> valuesInInitialState;
	
	protected Abstraction(TMWorklistBasedAnalysis worklistBasedAnalysis, Configuration config, Set<InstanceKey> valuesInInitialState) {
		this.analysis = worklistBasedAnalysis;
		this.configuration = config;
		this.valuesInInitialState = valuesInInitialState;
	}		
	
	public Set<Abstraction> transition(Unit node) {
		Stmt stmt = (Stmt) node;
		
        boolean mightHaveSideEffects = mightHaveSideEffects(stmt);

		Set<Configuration> newConfigs = new HashSet<Configuration>();
		Set<Shadow> shadows = analysis.getJob().shadowsOfStmt(stmt);

		Set<InstanceKey> outValuesInInitialState = new HashSet<InstanceKey>(valuesInInitialState);
		
		if(shadows.isEmpty() || configuration.isTainted()) {
			newConfigs.add(configuration);
		} else {		
			for(Shadow shadow: shadows) {
				Configuration newConfig = configuration.doTransition(shadow, valuesInInitialState);
	            newConfigs.add(newConfig);
	            //processValuesInInitialState(shadow,outValuesInInitialState);
			}
		}
		
        //processNewExpression(stmt,outValuesInInitialState);

        
        Set<Abstraction> outSet = new HashSet<Abstraction>();
        for (Configuration outConf : newConfigs) {
        	//taint outgoing value if necessary
        	if(mightHaveSideEffects) outConf = outConf.taint();
        	outSet.add(new Abstraction(analysis,outConf,outValuesInInitialState));
		}
        
		return outSet;
	}
	
//	/**
//     * At any assignment of a new-expression we know that the instance key representing that
//     * new-expression has to be in its initial state. We can exploit that for tracematches.
//     * This method analyses stmt. If it is an assign statement and the RHS is a new-expression
//     * then we add the instance key for that expression to valuesInInitialState. 
//     */
//    protected void processNewExpression(Stmt stmt, Set<InstanceKey> valuesInInitialState) {
//    	if(stmt instanceof AssignStmt) {
//			AssignStmt assignStmt = (AssignStmt) stmt;
//    		Value rightOp = assignStmt.getRightOp();
//    		if(rightOp instanceof NewExpr) {
//    			Local assignedLocal = (Local) assignStmt.getLeftOp();
//    			Stmt succ = (Stmt) analysis.getJob().unitGraph().getSuccsOf(stmt).get(0);//there is only one successor of any new-statement
//    			valuesInInitialState.add(new InstanceKey(assignedLocal,succ,analysis.getJob().method(),analysis.getJob().localMustAliasAnalysis(),analysis.getJob().localNotMayAliasAnalysis()));
//    		}
//    	}		
//	}
//    
//    /**
//     * This method removes an instance key k from valuesInInitialState in the case where shadow might have moved
//     * k out of the initial state. This is assumed to be the case if (1) the shadow's symbol is an initial
//     * symbol of the tracematch and (2) any instance keys referred to by the shadow may alias k.
//     */
//    protected void processValuesInInitialState(Shadow shadow, Set<InstanceKey> valuesInInitialState) {
//    	//FIXME: this is not correct in cases where the initial symbol does not bind all values
//    	if(analysis.getJob().initialShadowAdviceMethods().contains(shadow.getAdviceDecl().getImpl().getSootMethod())) {
//        	Map<String, InstanceKey> varToInstanceKey = this.analysis.reMap(shadow.getAdviceFormalToSootLocal());
//    		for (InstanceKey possiblyTransitionedInstanceKey : varToInstanceKey.values()) {
//				for (Iterator initialValuesIter = valuesInInitialState.iterator(); initialValuesIter.hasNext();) {
//					InstanceKey instanceKeyInInitialState = (InstanceKey) initialValuesIter.next();
//					if(!instanceKeyInInitialState.mayNotAlias(possiblyTransitionedInstanceKey)) {
//						initialValuesIter.remove();
//					}
//				}
//			}
//    	}
//	}
    
	protected boolean mightHaveSideEffects(Stmt s) {
	    Collection<Shadow> shadows = transitivelyCalledShadows(s);
		for (Shadow shadow : shadows) {
			if(analysis.getJob().overlappingSymbolShadows().contains(shadow)) {
				return true;
			}
		}
		return false;
	}
	
    /**
	 * Returns the collection of <code>Shadow</code>s triggered in transitive callees from <code>s</code>.
	 * @param s any statement
	 */
	protected Collection<Shadow> transitivelyCalledShadows(Stmt s) {
        HashSet<Shadow> shadows = new HashSet<Shadow>();
        HashSet<SootMethod> calleeMethods = new HashSet<SootMethod>();
        LinkedList<MethodOrMethodContext> methodsToProcess = new LinkedList();

        // Collect initial edges out of given statement in methodsToProcess
        Iterator<Edge> initialEdges = analysis.getJob().abstractedCallGraph().edgesOutOf(s);
        while (initialEdges.hasNext()) {
            Edge e = initialEdges.next();
            methodsToProcess.add(e.getTgt());
            calleeMethods.add(e.getTgt().method());
        }

        // Collect transitive callees of methodsToProcess
        while (!methodsToProcess.isEmpty()) {
            MethodOrMethodContext mm = methodsToProcess.removeFirst();
            Iterator mIt = analysis.getJob().abstractedCallGraph().edgesOutOf(mm);

            while (mIt.hasNext()) {
                Edge e = (Edge)mIt.next();
                if (!calleeMethods.contains(e.getTgt().method())) {
                    methodsToProcess.add(e.getTgt());
                    calleeMethods.add(e.getTgt().method());
                }
            }
        }

        // Collect all shadows in calleeMethods
        for (SootMethod method : calleeMethods) {
        	Set<Shadow> possiblyEnabledTMShadowsOf = analysis.getJob().possiblyEnabledTMShadowsOf(method);
        	for (Shadow shadow : possiblyEnabledTMShadowsOf) {
				if(shadow.isEnabled()) {
					shadows.add(shadow);
				}
			}
        	
        }
        return shadows;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((analysis == null) ? 0 : analysis.hashCode());
		result = prime * result
				+ ((configuration == null) ? 0 : configuration.hashCode());
		result = prime
				* result
				+ ((valuesInInitialState == null) ? 0 : valuesInInitialState
						.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Abstraction other = (Abstraction) obj;
		if (analysis == null) {
			if (other.analysis != null)
				return false;
		} else if (!analysis.equals(other.analysis))
			return false;
		if (configuration == null) {
			if (other.configuration != null)
				return false;
		} else if (!configuration.equals(other.configuration))
			return false;
		if (valuesInInitialState == null) {
			if (other.valuesInInitialState != null)
				return false;
		} else if (!valuesInInitialState.equals(other.valuesInInitialState))
			return false;
		return true;
	}
    
    @Override
    public String toString() {
    	StringBuffer sb = new StringBuffer();
    	sb.append("Config: \n");
    	sb.append(configuration.toString());
    	sb.append("Values in initial state: \n");
    	sb.append(valuesInInitialState);
    	sb.append("\n");
    	return sb.toString();
    }

	public Configuration getConfiguration() {
		return configuration;
	}
}